@@ -105,3 +105,40 @@ else {
 }
 return(assigndata)
 }
+
+make_strata_encoder <- function(levels_list) {
+  stopifnot(is.list(levels_list), length(levels_list) >= 1)
+
+  key <- do.call(expand.grid, c(levels_list, KEEP.OUT.ATTRS = FALSE, stringsAsFactors = FALSE))
+  key$stratum_id <- seq_len(nrow(key))
+
+  L <- vapply(levels_list, length, integer(1))
+  offsets <- rev(cumprod(c(rev(L[-1]), 1)))
+
+
+  index_in_levels <- function(vals, levels_list) {
+    mapply(function(v, lv) {
+      idx <- match(v, lv)
+      if (is.na(idx)) stop(sprintf("value %s is not in level %s ", v, paste(lv, collapse = ",")))
+      idx
+    }, vals, levels_list, SIMPLIFY = TRUE, USE.NAMES = FALSE)
+  }
+
+
+  encode_df <- function(df) {
+    stopifnot(is.data.frame(df))
+    merged <- merge(df, key, by = names(levels_list), all.x = TRUE, sort = FALSE)
+    if (anyNA(merged$stratum_id)) {
+      stop("It has been found that the values of some rows are not within the preset range. Please check the input.")
+    }
+    merged
+  }
+
+  list(
+    levels_list = levels_list,
+    key = key,
+    encode_df = encode_df
+
+  )
+}
+
